﻿using AuthenticationServices;
using Foundation;
using KerberosSSOA.Services;
using System;

namespace KerberosSSOA.iOS
{
    public class AuthenticationService : ASAuthorizationControllerDelegate, IAuthenticationService
    {
        public bool IsSignedIn { get; private set; }

        public event EventHandler OnSignedInChanged;

        public AuthenticationService()
        {
            // Change <kerberos-realm> to the realm of your Kerberos installation
            m_AuthProvider = ASAuthorizationSingleSignOnProvider.CreateProvider(new NSUrl("realm://<kerberos-realm>"));
        }

        public void SignIn(object sender)
        {
            
            if (m_AuthProvider.CanPerformAuthorization)
            {
                var request = m_AuthProvider.CreateRequest();
                request.RequestedOperation = ASAuthorizationOperation.Login;
                m_AuthController = new ASAuthorizationController(new[] { request });
                m_AuthController.Delegate = this;
                m_AuthController.PresentationContextProvider = sender as IASAuthorizationControllerPresentationContextProviding;
                m_AuthController.PerformRequests();
            }
        }

        public override void DidComplete(ASAuthorizationController controller, ASAuthorization authorization)
        {
            var request = controller.AuthorizationRequests[0] as ASAuthorizationSingleSignOnRequest;

            if (request.RequestedOperation == ASAuthorizationOperation.Login)
            {
                var credential = authorization.GetCredential<ASAuthorizationSingleSignOnCredential>();
                var response = credential.AuthenticatedResponse;

                if (response.StatusCode == 200)
                {
                    IsSignedIn = true;
                }
                else
                {
                    IsSignedIn = false;
                }

                OnSignedInChanged?.Invoke(this, null);
            }
            else if (request.RequestedOperation == ASAuthorizationOperation.Logout)
            {
                IsSignedIn = false;

                OnSignedInChanged?.Invoke(this, null);
            }
        }

        public void SignOut(object sender)
        {
            if (m_AuthProvider.CanPerformAuthorization)
            {
                var request = m_AuthProvider.CreateRequest();
                request.RequestedOperation = ASAuthorizationOperation.Logout;
                m_AuthController = new ASAuthorizationController(new[] { request });
                m_AuthController.Delegate = this;
                m_AuthController.PresentationContextProvider = sender as IASAuthorizationControllerPresentationContextProviding;
                m_AuthController.PerformRequests();
            }
        }

        public override void DidComplete(ASAuthorizationController controller, NSError error)
        {
            Console.WriteLine("ERROR");

            IsSignedIn = false;
            OnSignedInChanged?.Invoke(this, null);
        }

        private ASAuthorizationController m_AuthController;
        private readonly ASAuthorizationSingleSignOnProvider m_AuthProvider;
    }
}