//
//  APICaller.swift
//  DIPSAuthPrototypeA
//
//  Created by trd-studs on 25/03/2021.
//

import AppAuth
import Foundation
import JWTDecode

class APICaller {
    static var shared = APICaller()
    
    
    func get(urlRequest: URLRequest, completion: @escaping (Data?, HTTPURLResponse, URLRequest) -> Void) {
        let currentAccessToken: String? = AuthManager.shared.authState?.lastTokenResponse?.accessToken
        
        AuthManager.shared.authState?.performAction() {
            accessToken, idToken, error in
            
            guard error == nil else {
                print(error?.localizedDescription)
                return
            }
            
            guard let accessToken = accessToken else {
                return
            }
            
            if currentAccessToken != accessToken {
                print("Access token was refreshed automatically.")
            } else {
                print("Access token was fresh and not updated.")
            }
            
            print(accessToken)
            
            var jwt: JWT?
            do {
                
                jwt = try decode(jwt: accessToken)
            } catch {
                print("Oh no :(")
                return
            }
            
            let ticket = jwt?.body["dips-ticket"] as? String
            
            var urlRequest = urlRequest
            
            var requestHeaders = urlRequest.allHTTPHeaderFields ?? [:]
            //requestHeaders["Authorization"] = "Bearer \(accessToken)"
            requestHeaders["ticketheader"] = ticket
            urlRequest.allHTTPHeaderFields = requestHeaders
            
            self.sendUrlRequest(urlRequest: urlRequest) {
                data, response, request in
                
                guard let data = data, data.count > 0 else {
                    print ("HTTP response data is empty")
                    return
                }
                
                if (response.statusCode != 200) {
                    let responseText: String? = String(data: data, encoding: String.Encoding.utf8)
                    
                    if (response.statusCode == 401) {
                        var json: [AnyHashable: Any]?
                        
                        do {
                            json = try JSONSerialization.jsonObject(with: data, options: []) as? [String: Any]
                        } catch {
                            print("JSON serialization error.")
                        }
                        
                        let oauthError = OIDErrorUtilities.resourceServerAuthorizationError(withCode: 0, errorResponse: json, underlyingError: nil)
                        
                        AuthManager.shared.authState?.update(withAuthorizationError: oauthError)
                    }
                }
                
                completion(data, response, urlRequest)
            }
        }
    }
    
    private func sendUrlRequest(urlRequest: URLRequest, completion: @escaping (Data?, HTTPURLResponse, URLRequest) -> Void) {
        let task = AuthManager.shared.customUrlSession.dataTask(with: urlRequest) {
            data, response, error in
            DispatchQueue.main.async {
                guard error == nil else {
                    return
                }
                
                guard let response = response as? HTTPURLResponse else {
                    return
                }
                
                completion(data, response, urlRequest)
            }
        }
        
        task.resume()
    }
}
